
setwd("SET_FILE_LOCATION")

library(dplyr)
library(ggplot2)
library(ncdf4)
library(readr)
library(tidyr)
library(purrr)
library(stringr)
library(scico)
library(hetGP)
library(RobustGaSP)

theme_set(theme_bw())
source("_extendedLHC.R")

# ------------------------------------------------------------------------------
# ------------------------------ Clean up data ---------------------------------
# ------------------------------------------------------------------------------

nd_info_raw <- read_csv("data/nd_OAT_ensemble.csv")
names(nd_info_raw) <- c("exp", "source", "rs", "inventory", "flux", "residence")

nd_scaled <- nd_info_raw %>%
  mutate(
    source = (source - min(nd_info_raw$source)) / 
    (max(nd_info_raw$source) - min(nd_info_raw$source)),
    rs = (rs - min(nd_info_raw$rs)) / 
    (max(nd_info_raw$rs) - min(nd_info_raw$rs)),
    inventory = inventory/(10^12),
    flux = flux/(10^9),
    residence = residence/(10^3)
  )

ggplot(nd_scaled) +
  geom_point(aes(x = rs, y = source, colour = inventory), size = 2) +
  scale_colour_scico(
    direction = 1, palette = "batlow", limits = c(0,18)
  )

emulator <- rgasp(
  design = nd_scaled %>% dplyr::select(source, rs) %>% as.matrix(),
  response = nd_scaled %>% dplyr::select(inventory) %>% as.matrix(),
  trend = cbind(
    matrix(rep(1, nrow(nd_scaled))),
    nd_scaled %>% dplyr::select(source, rs) %>% as.matrix()
  )
)

prediction <- predict(
  emulator,
  expand.grid(seq(0, 1, length = 100), seq(0, 1, length = 100)) %>% as.matrix(),
  cbind(
    matrix(rep(1, 100^2)),
    expand.grid(seq(0, 1, length = 100), seq(0, 1, length = 100))  %>% as.matrix()
  )
)

expand.grid(
  source = seq(0, 1, length = 100), 
  rs = seq(0, 1, length = 100)
) %>%
  as_tibble() %>%
  mutate(
    mean = prediction$mean,
    sd = prediction$sd,
    mean = ifelse(mean < 0, 0, mean)
  ) %>%
  ggplot() +
    geom_tile(
      aes(
        x = rs, y = source, width = 0.01011, 
        height = 0.01011, fill = mean
      )
    ) +
    geom_point(
      data = nd_scaled, 
      mapping = aes(x = rs, y = source), 
      size = 2
    ) +
    scale_fill_scico(direction = 1, palette = "batlow", limits = c(0,18)) +
    scale_x_continuous("rs", expand = c(0, 0)) +
    scale_y_continuous("source", expand = c(0, 0))

expand.grid(
  source = seq(0, 1, length = 100), 
  rs = seq(0, 1, length = 100)
) %>%
  as_tibble() %>%
  mutate(
    mean = prediction$mean,
    sd = prediction$sd,
    lower = mean - 2 * sd,
    upper = mean + 2 * sd
  ) %>%
  filter(upper > 3, lower < 5) %>%
  ggplot() +
    geom_tile(
      aes(
        x = rs, y = source, width = 0.01011, 
        height = 0.01011, fill = mean
      )
    ) +
    geom_point(
      data = nd_scaled, 
      mapping = aes(x = rs, y = source), 
      size = 2
    ) +
    scale_fill_scico(direction = 1, palette = "batlow", limits = c(0,18)) +
    scale_x_continuous("rs", expand = c(0, 0)) +
    scale_y_continuous("source", expand = c(0, 0))

# ------------------------------------------------------------------------------
# ------------------------------- Design ---------------------------------------
# ------------------------------------------------------------------------------

len <- 100

design_pred <- predict(
  emulator,
  expand.grid(seq(0, 1, length = len), seq(0, 1, length = len)) %>% as.matrix(),
  cbind(
    matrix(rep(1, len^2)),
    expand.grid(seq(0, 1, length = len), seq(0, 1, length = len))  %>% as.matrix()
  )
)

grid_small <- expand.grid(
  source = seq(0, 1, length = len), 
  rs = seq(0, 1, length = len)
) %>%
  as_tibble() %>%
  mutate(
    mean = design_pred$mean,
    sd = design_pred$sd,
    lower = mean - 2 * sd,
    upper = mean + 2 * sd
  ) %>%
  filter(upper > 3, lower < 5)

TestDesign <- sim.anneal(
  D_candidates = as.matrix(grid_small %>% dplyr::select(source, rs)), 
  RestDesign = as.matrix(nd_scaled %>% dplyr::select(source, rs)),
  n=8, w=2, t0=1000, FAC_t=0.7, Imax = 500, p=50, range.space=c(-1,1)
)

final_design <- as.matrix(grid_small %>% dplyr::select(source, rs))[TestDesign, ] %>%
  as_tibble()

expand.grid(
  source = seq(0, 1, length = 100), 
  rs = seq(0, 1, length = 100)
) %>%
  as_tibble() %>%
  mutate(
    mean = prediction$mean,
    sd = prediction$sd,
    lower = mean - 2 * sd,
    upper = mean + 2 * sd
  ) %>%
  filter(upper > 3, lower < 5) %>%
  ggplot() +
    geom_tile(
      aes(
        x = rs, y = source, width = 0.01011, 
        height = 0.01011, fill = mean
      )
    ) +
    geom_point(
      data = nd_scaled, 
      mapping = aes(x = rs, y = source), 
      size = 2
    ) +
    geom_point(
      data = final_design, 
      mapping = aes(x = rs, y = source), 
      size = 2, colour = "red"
    ) +
    scale_fill_scico("nd", direction = 1, palette = "batlow", limits = c(0,18)) +
    scale_x_continuous("rs", expand = c(0, 0)) +
    scale_y_continuous("source", expand = c(0, 0))

final_design %>%
  mutate(
    source = source * (max(nd_info_raw$source) - min(nd_info_raw$source)) + 
      min(nd_info_raw$source),
    rs = rs * (max(nd_info_raw$rs) - min(nd_info_raw$rs)) + 
      min(nd_info_raw$rs)
  ) %>%
  write.csv("wave1_design.csv")