
setwd("SET_FILE_LOCATION")

library(dplyr)
library(ggplot2)
library(ncdf4)
library(tidyr)
library(purrr)
library(stringr)
library(scico)
library(hetGP)
library(RobustGaSP)
library(readr)
library(plotly)

theme_set(theme_bw())

source("_extendedLHC.R")

# ------------------------------------------------------------------------------
# -------------------------- Design wave 2 emulator ----------------------------
# ------------------------------------------------------------------------------

# Leaving scaling as a parameter

nd_info_raw <- read_csv("data/wave2_results.csv") %>%
  mutate(
    mae_end = (mae_end - min(mae_end)) / 
    (max(mae_end) - min(mae_end)),
    mae_nd = (mae_nd - min(mae_nd)) / 
    (max(mae_nd) - min(mae_nd))) %>%
  mutate(metric = 1 * mae_end + 1 * mae_nd) %>% #Change metric in here to what you're wanting
  dplyr::select(exp, sed, rs_tune, scaling, nd_inv, residence, metric) 

nd_scaled <- nd_info_raw %>%
  mutate(
    sed = (sed - min(nd_info_raw$sed)) / 
    (max(nd_info_raw$sed) - min(nd_info_raw$sed)),
    rs_tune = (rs_tune - min(nd_info_raw$rs_tune)) / 
    (max(nd_info_raw$rs_tune) - min(nd_info_raw$rs_tune)),
    scaling = (scaling - 1) / 3,
    nd_inv = nd_inv/(10^12),
    residence = residence/(10^3)
  )

(0.0045 - min(nd_info_raw$rs_tune)) / 
    (max(nd_info_raw$rs_tune) - min(nd_info_raw$rs_tune))

nd_emulator <- rgasp(
  design = nd_scaled %>% 
    dplyr::select(sed, rs_tune, scaling) %>% 
    as.matrix(),
  response = nd_scaled %>% 
    dplyr::select(nd_inv) %>% 
    as.matrix(),
  trend = cbind(
    matrix(rep(1, nrow(nd_scaled))),
    nd_scaled %>% dplyr::select(sed, rs_tune, scaling) %>% as.matrix()
  )
)

residence_emulator  <- rgasp(
  design = nd_scaled %>% 
    dplyr::select(sed, rs_tune, scaling) %>% 
    as.matrix(),
  response = nd_scaled %>% 
    dplyr::select(residence) %>% 
    as.matrix(),
  trend = cbind(
    matrix(rep(1, nrow(nd_scaled))),
    nd_scaled %>% dplyr::select(sed, rs_tune, scaling) %>% as.matrix()
  )
)

metric_emulator  <- rgasp(
  design = nd_scaled %>% 
    dplyr::select(sed, rs_tune, scaling) %>% 
    as.matrix(),
  response = nd_scaled %>% 
    dplyr::select(metric) %>% 
    as.matrix(),
  trend = cbind(
    matrix(rep(1, nrow(nd_scaled))),
    nd_scaled %>% dplyr::select(sed, rs_tune, scaling) %>% as.matrix()
  )
)

n <- 100

pred_matrix <- expand.grid(
  seq(0, 1, length = n),
  seq(0, 1, length = n),
  seq(0, 1, length = n)
) %>% as.matrix()

nd_prediction <- predict(
  nd_emulator,
  pred_matrix,
  cbind( matrix(rep(1, n^3)), pred_matrix )
)

residence_prediction <- predict(
  residence_emulator,
  pred_matrix,
  cbind( matrix(rep(1, n^3)), pred_matrix )
)

metric_prediction <- predict(
  metric_emulator,
  pred_matrix,
  cbind( matrix(rep(1, n^3)), pred_matrix )
)

plot_points <- expand.grid(
  sed = seq(0, 1, length = n), 
  rs_tune = seq(0, 1, length = n),
  scaling = seq(0, 1, length = n)
) %>%
  as_tibble() %>%
  mutate(
    nd_mean = nd_prediction$mean,
    nd_sd = nd_prediction$sd,
    nd_lower = nd_mean - 2 * nd_sd,
    nd_upper = nd_mean + 2 * nd_sd,
    res_mean = residence_prediction$mean,
    res_sd = residence_prediction$sd,
    res_lower = res_mean - 2 * res_sd,
    res_upper = res_mean + 2 * res_sd,
    metric = metric_prediction$mean
  ) %>%
  filter(
    rs_tune < 0.7, nd_upper > 3.6, nd_lower < 4.6, 
    res_mean < 1
  )

expand.grid(
    sed = seq(0, 1, length = n), 
    rs_tune = seq(0, 1, length = n),
    scaling = seq(0, 1, length = n)
  ) %>%
    as_tibble() %>%
    mutate(
      nd_mean = nd_prediction$mean,
      nd_sd = nd_prediction$sd,
      nd_lower = nd_mean - 2 * nd_sd,
      nd_upper = nd_mean + 2 * nd_sd,
      res_mean = residence_prediction$mean,
      res_sd = residence_prediction$sd,
      res_lower = res_mean - 2 * res_sd,
      res_upper = res_mean + 2 * res_sd,
      metric = metric_prediction$mean
    )%>%
  filter(
    scaling == 0, nd_upper > 3.6, nd_lower < 4.6, 
    res_upper > 0.35, res_lower < 0.90, rs_tune < 0.7
  ) %>%
  ggplot() +
    geom_tile(
      aes(
        x = rs_tune, y = sed, width = 1/(n-1), 
        height = 1/(n-1), fill = metric
      )
    ) +
    scale_x_continuous("rs", expand = c(0, 0), limits = c(0,1)) +
    scale_y_continuous("source", expand = c(0, 0), limits = c(0,1))

fig <- plot_ly(
  as.data.frame(plot_points), 
  x = ~rs_tune, 
  y = ~sed, 
  z = ~scaling,
  marker = list(color = ~metric, colorscale = c('#683531', '#FFE1A1'), showscale = TRUE)
)

fig <- fig %>% add_markers()

fig

plot_points[which.min(plot_points$metric), ] %>%
  mutate(
    sed = sed * (max(nd_info_raw$sed) - min(nd_info_raw$sed)) + 
      min(nd_info_raw$sed),
    rs_tune = rs_tune * (max(nd_info_raw$rs_tune) - min(nd_info_raw$rs_tune)) + 
      min(nd_info_raw$rs_tune),
    scaling = scaling * 3 + 1
  ) %>%
  dplyr::select(rs_tune, sed, res_mean, nd_mean, metric, everything())
